///////////////////////////////////////////////////////////
//  FF_LPR-SDK.h
//  Created on:      02-Apr-2015 23:10:34
//  Author:          v.tarasyuk@ff-kb.com
//  Revision         6
///////////////////////////////////////////////////////////
#ifndef __FF_LPR_SDK_H__
#define __FF_LPR_SDK_H__

#define FF_MAX_POLYGON_POINTS       4
#define FF_MAX_ROI_COUNT            4
#define FF_MAX_RESULTS_COUNT       16
#define FF_MAX_PLATE_NUMBER_LENGTH 32
#define FF_COUNTRY_CODE_LENGTH      4

#define FF_DEFAULT_THRESHOLD_CONFIDENCE  (0.6f)

#define FF_RESERVED_SIZE         1024

#define FF_TEXT_FIELD_LENGTH      128

#define FF_LPR_NOERROR              0

#define FF_LPR_INVALID_IMAGE       -1 // Invalid image passed in parameters ( w * h * 3 / 2 != sz )
#define FF_LPR_NO_ONE_ROI          -2 // No one enabled ROI passed in parameters
#define FF_LPR_INVALID_ROI         -3 // ROI and min/max plate width isn't valid
#define FF_LPR_LICENSE_LIMIT       -4 // licensed separate running recognizers value reached
#define FF_LPR_HANDLE_BUSY         -5 // current handle is busy
#define FF_LPR_CREATE_TIMEOUT      -6 // timeout, it is not enough processors resources or concurrent creating handles from many threads
#define FF_LPR_INVALID_HANDLE      -7 //
#define FF_LPR_HANDLE_NONEXIST     -8 //
#define FF_LPR_TIMEOUT_ERROR       -9 //
#define FF_LPR_PARAMS_ERROR       -10 //

#define FF_LPR_UNREGISTERED      -252 // unregistered
#define FF_LPR_REGISTERED        -253 // already registered
#define FF_LPR_REGISTRY_ERROR    -254 // error write to registry ( HKCU\\SOFTWARE branch )

#define FF_LPR_INTERNAL_ERROR    -255 // if error call or error params

// License system errors
#define FF_ERR_SERIAL_CORRUPTED			0x0001
#define FF_ERR_SERIAL_INVALID			0x0002
#define FF_ERR_SERIAL_BLACKLISTED		0x0004
#define FF_ERR_SERIAL_DATE_EXPIRED		0x0008
#define FF_ERR_SERIAL_RUNNING_TIME_OVER	0x0010
#define FF_ERR_SERIAL_BAD_HWID			0x0020
#define FF_ERR_SERIAL_MAX_BUILD_EXPIRED	0x0040

#define FF_ERR_1_SMALL_BUFFER             1001
#define FF_ERR_1_NO_CONNECTION            1002
#define FF_ERR_1_BAD_REPLY                1003
#define FF_ERR_1_BANNED                   1004
#define FF_ERR_1_CORRUPTED                1005
#define FF_ERR_1_BAD_CODE                 1006
#define FF_ERR_1_ALREADY_USED             1007
#define FF_ERR_1_SERIAL_UNKNOWN           1008
#define FF_ERR_1_EXPIRED                  1009

#define FF_ERR_BROKEN_DATA                1202 // invalid license data
#define FF_ERR_LICENSE_NOT_FOUND          1203 // License information was not found ater loading DLL

// Default values
#define FF_SDK_MIN_PLATE_WIDTH             100
#define FF_SDK_MIN_CHARS_COUNT               3
#define FF_SDK_MAX_CHARS_COUNT              12

//
// Types definition
//
typedef ULONG_PTR LPR_HANDLE;

//
// predefines
//
typedef struct FF_Lpr_Version
{
    unsigned int MaxConcurrentHandles;
    char         SdkVersion[FF_TEXT_FIELD_LENGTH];
    char         Version[FF_TEXT_FIELD_LENGTH];
    char         Copyright[FF_TEXT_FIELD_LENGTH];
    unsigned int UsageValue;
    /**
     * @brief future_reserved - reserved size for future
     */
    char          future_reserved[FF_RESERVED_SIZE - 2];
} FF_Lpr_Version, *PFF_Lpr_Version;

/**
 * @brief
 *  Simple point
 *  x, y - coordinates, points
 * ( 0, 0 ) - the left top point of image
*/
typedef struct FF_Point
{
    int x;
    int y;
} FF_Point;

/**
 * @brief
 *   Simple polygon
*/
typedef struct FF_Polygon
{
    FF_Point pts[FF_MAX_POLYGON_POINTS];
    FF_Polygon()
    {
        for ( int i = 0; i < FF_MAX_POLYGON_POINTS; ++i )
        {
            pts[i].x = 0;
            pts[i].y = 0;
        }
    }
} FF_Polygon;

typedef struct FF_Color
{
    unsigned char r;
    unsigned char g;
    unsigned char b;
    float confidence;
    FF_Color()
    {
        r = g = b = 0;
        confidence = 0.0f;
    }
    FF_Color( unsigned char _r, unsigned char _g, unsigned char _b, float _confidence ) :
        r( _r ), g( _g ), b( _b ), confidence( _confidence )
    {
    }
} FF_Color;

/**
 * @brief
 * Input image parameters
*/
typedef struct FF_Image
{
    unsigned char * data;      /// @brief yuv12data image data including U and V components
    unsigned long   size;      /// @brief yuv12data image data size in bytes
    /**
     * @brief
     *     width  - image width,  pixels
     *     height - image height, pixels
     *     ( width * height ) must be aligned to 16
     */
    unsigned int    width;
    unsigned int    height;
    /**
     * @brief
     *     Default constructor
     */
    FF_Image()
    {
        data   = 0;
        size   = 0;
        width  = 0;
        height = 0;
    }
    unsigned int uvOffset() { return ( width * height ); }
} FF_Image;

/**
 *
 */
typedef struct FF_RecognizeResult
{
    /// ASCIIZ text of recognized plate
    char PlateText[FF_MAX_PLATE_NUMBER_LENGTH];
	/// wide char text of recognized plate
	wchar_t wPlateText[FF_MAX_PLATE_NUMBER_LENGTH];
    /**
     * @brief confidence - confidence of plate recognition
     */
    float confidence;
    /**
     * @brief 3 letters country code, if detected
     */
    char countryCode[FF_COUNTRY_CODE_LENGTH];

    /**
     * @brief Coordinates of plate, start from top left corner of image
     */
    ///
    FF_Polygon coordinates;

    /**
     * @brief color - Color of plate background
     */
    FF_Color color;

	/**
	 * @brief roi - roi number where number was recognized
	 */
	int roi;

    /**
     * @brief future_reserved - reserved size for future
     */
    char future_reserved[FF_RESERVED_SIZE];
} FF_RecognizeResult, *PFF_RecognizeResult;

/**
 * @brief input parameters
 */
typedef struct FF_LprParameters
{
    /**
     * @brief Image data
     */
    FF_Image  yuv12image;
    /**
     * @brief Regions of interest
     */
    FF_Polygon ROIs[FF_MAX_ROI_COUNT];
    /**
     * @brief valid Count of Rects of interest ( ROIs )
     */
    int           ROIsCount;
    /**
     * @brief Recognize limitations
     * minPlateWidth            - minimum width of a standart one row plate, pixels, default 100
     * maxPlateWidth            - maximum allowed plate width, pixels, default 100 * 3
     * minSymbolCount           - minimum count of symbol in text - for a valid plate, default  4
     * maxSymbolCount           - maximum count of symbol in text - for a valid plate, default 12
     * plateThresholdConfidence - a minimal confidence for a valid plate, default 0.6
     */
    unsigned int  minPlateWidth;
    unsigned int  maxPlateWidth;
    unsigned int  minSymbolCount;
    unsigned int  maxSymbolCount;
    float         plateThresholdConfidence;
    /**
     * @brief User parameter.
     */
    void        * userParam;
    /**
     * @brief future_reserved - reserved size for future
     */
    char          future_reserved[FF_RESERVED_SIZE];
    FF_LprParameters()
    {
        ROIsCount                = 0;
        minPlateWidth            = FF_SDK_MIN_PLATE_WIDTH;
        maxPlateWidth            = FF_SDK_MIN_PLATE_WIDTH * 3;
        minSymbolCount           = FF_SDK_MIN_CHARS_COUNT;
        maxSymbolCount           = FF_SDK_MAX_CHARS_COUNT;
        plateThresholdConfidence = FF_DEFAULT_THRESHOLD_CONFIDENCE;
    }
} FF_LprParameters, *PFF_LprParameters;

/**
 * @brief result parameters
 */
typedef struct FF_LprResult
{
    /**
     * @brief result parameters
     * results      - ( out ) array of FF_RecognizeResult allocated with malloc
     * resultsCount - ( out ) recognized results count in the array
     */
    FF_RecognizeResult results[FF_MAX_RESULTS_COUNT];
    unsigned int       resultsCount;
    /**
     * @brief User parameter. If user will make synthetic asynchronous mode
     */
    void * userParam;
    /**
     * @brief future_reserved - reserved size for future
     */
    char future_reserved[FF_RESERVED_SIZE];
} FF_LprResult, *PFF_LprResult;

/**
* function prototypes
**/
/**
/* FFSdkRegister prototype
* @brief Register LPR SDK with license code obtained after purchasing
* @param license_code     [in]  - license code obtained after purchasing
* check ff_error value after return.
**/
typedef void( *pfnFFSdkRegister )( const char * license_code, LONG * ff_error );
/* FFSdkUnregister prototype
* @param license_code     [in]  - license code obtained after purchasing
* check ff_error value after return.
*/
typedef void( *pfnFFSdkUnregister )( const char * license_code, LONG * ff_error );
/**
* GetVersion prototype
* @brief Get LPR version
* @param version structure FF_Lpr_Version [in/out]
* @param ff_error  [out] - this value points to error, look at FF_ERR_SERIAL_*
**/
typedef void( *pfnFFSdkGetVersion )( FF_Lpr_Version &, LONG * ff_error );
/**
* FFSdkCreateLpr prototype
* @brief Create LPR handle
* @param lprHandle [out] - handle of lpr process or 0 if error
* @param ff_error  [out] - value points to error, look at FF_ERR_SERIAL_*
**/
typedef void( *pfnFFSdkCreateLpr )( LPR_HANDLE * lprHandle, LONG * ff_error );
/**
* FFSdkRecognizeImage prototype
**/
typedef void( *pfnFFSdkRecognizeImage )( LPR_HANDLE, FF_LprParameters &, FF_LprResult *, LONG * ff_error );
/**
* FFSdkDestroyLpr prototype
**/
typedef void( *pfnFFSdkDestroyLpr )( LPR_HANDLE );

#endif // __FF_LPR_SDK_H__
